import asyncio

from .event_loop import EventLoop

__all__ = ["EventLoopPolicy"]


class EventLoopPolicy(asyncio.DefaultEventLoopPolicy):  # type: ignore
    """
    An asyncio event loop policy with the all the default behaviours except
    that it uses the `asyncio_gevent.EventLoop` which uses gevent for
    scheduling
    """

    _loop_factory = EventLoop



import asyncio
from asyncio import CancelledError
from typing import Callable
from typing import Optional

import gevent.event

__all__ = ["future_to_greenlet"]


def future_to_greenlet(
    future: asyncio.Future,
    loop: Optional[asyncio.AbstractEventLoop] = None,
    autostart_future: bool = True,
    autocancel_future: bool = True,
    autokill_greenlet: bool = True,
) -> gevent.Greenlet:
    """
    Wrap a future in a greenlet.

    The greenlet returned by this function will not start automatically, so you
    need to call `Greenlet.start()` manually.

    If `future` is a coroutine object, it will be scheduled as a task on the
    `loop` when the greenlet starts. If no `loop` argument has been passed, the
    running event loop will be used. If there is no running event loop, a new
    event loop will be started using the current event loop policy.

    If the future is not already scheduled, then it won't be scheduled for
    execution until the greenlet starts running. To prevent the future from
    being scheduled automatically, you can pass `autostart_future=False` as an
    argument to `future_to_greenlet`.

    If the greenlet gets killed, the by default the future gets cancelled. To
    prevent this from happening and having the future return the
    `GreenletExit` objct instead, you can pass `autocancel_future=False` as an
    argument to `future_to_greenlet`.

    If the future gets cancelled, the greenlet gets killed and will return a
    `GreenletExit`. This default behaviour can be circumvented by passing
    `autokill_greenlet=False` and the greenlet will raise the `CancelledError`
    instead.
    """

    def on_cancelled():
        if autokill_greenlet:
            greenlet.kill()
        else:
            raise

    greenlet = gevent.Greenlet(
        run=_run,
        future=future,
        loop=loop,
        autostart_future=autostart_future,
        on_cancelled=on_cancelled,
    )

    def cb(gt):
        if isinstance(gt.value, gevent.GreenletExit):
            future.cancel()

    if autocancel_future:
        greenlet.link_value(cb)

    return greenlet


def _run(
    future: asyncio.Future,
    loop: Optional[asyncio.AbstractEventLoop],
    autostart_future: bool,
    on_cancelled: Callable,
):
    active_loop = loop

    # If not loop argument was specified, try and use the running loop

    if not active_loop:
        try:
            active_loop = asyncio.get_running_loop()
        except RuntimeError:
            pass

    try:
        ensured_future: asyncio.Future

        if not autostart_future:
            ensured_future = future
        elif not active_loop:
            # If there's no running loop and no loop argument was specified,
            # then get a loop and run it to completion in a spawned greenlet

            active_loop = asyncio.new_event_loop()
            asyncio.set_event_loop(active_loop)

            ensured_future = asyncio.ensure_future(future, loop=active_loop)

            run_until_complete_greenlet = gevent.spawn(
                active_loop.run_until_complete, ensured_future
            )
            run_until_complete_greenlet.join()
        else:
            # If there's a running loop already or a loop argument was specified,
            # then schedule the future and block until it's done

            ensured_future = asyncio.ensure_future(future, loop=active_loop)

            event = gevent.event.Event()

            def done(_):
                event.set()

            ensured_future.add_done_callback(done)

            event.wait()

        return ensured_future.result()
    except CancelledError:
        on_cancelled()



import sys
import time

from gevent._interfaces import ILoop
from zope.interface import implementer

from .async_watcher import AsyncWatcher
from .callback import Callback
from .child_watcher import ChildWatcher
from .fork_watcher import ForkWatcher
from .io_watcher import IoWatcher
from .monkey_jail import MonkeyJail
from .signal_watcher import SignalWatcher
from .timer_watcher import TimerWatcher

__all__ = ["GeventLoop"]


@implementer(ILoop)
class GeventLoop:
    """

    The methods that create event loop watchers are `io`, `timer`,
    `signal`, `idle`, `prepare`, `check`, `fork`, `async_`, `child`,
    `stat`. These all return various types of :class:`IWatcher`.

    All of those methods have one or two common arguments. *ref* is a
    boolean saying whether the event loop is allowed to exit even if
    this watcher is still started. *priority* is event loop specific.
    """

    MAXPRI = 0

    def __init__(self, flags=None, default=None):
        self._aio = None
        self.error_handler = None
        self.fork_watchers = set()
        self._ref_count = 0
        self._stop_handle = None
        self.aio.set_exception_handler(self._handle_aio_error)

    def run(self, nowait=False, once=False):
        """
        Run the event loop.

        This is usually called automatically by the hub greenlet, but
        in special cases (when the hub is *not* running) you can use
        this to control how the event loop runs (for example, to integrate
        it with another event loop).
        """
        if self.aio.is_running:
            return
        self.aio.run_forever()

    def now(self):
        """
        now() -> float

        Return the loop's notion of the current time.

        This may not necessarily be related to :func:`time.time` (it
        may have a different starting point), but it must be expressed
        in fractional seconds (the same *units* used by :func:`time.time`).
        """
        return time.time()

    def update_now(self):
        """
        Update the loop's notion of the current time.

        .. versionadded:: 1.3
           In the past, this available as ``update``. This is still available as
           an alias but will be removed in the future.
        """
        pass

    def destroy(self):
        """
        Clean up resources used by this loop.

        If you create loops
        (especially loops that are not the default) you *should* call
        this method when you are done with the loop.

        .. caution::

            As an implementation note, the libev C loop implementation has a
            finalizer (``__del__``) that destroys the object, but the libuv
            and libev CFFI implementations do not. The C implementation may change.

        """
        pass

    def io(self, fd, events, ref=True, priority=None):
        """
        Create and return a new IO watcher for the given *fd*.

        *events* is a bitmask specifying which events to watch
        for. 1 means read, and 2 means write.
        """
        return IoWatcher(self, fd, events, ref, priority)

    def closing_fd(self, fd):
        """
        Inform the loop that the file descriptor *fd* is about to be closed.

        The loop may choose to schedule events to be delivered to any active
        IO watchers for the fd. libev does this so that the active watchers
        can be closed.

        :return: A boolean value that's true if active IO watchers were
           queued to run. Closing the FD should be deferred until the next
           run of the eventloop with a callback.
        """
        pass

    def timer(self, after, repeat=0.0, ref=True, priority=None):
        """
        Create and return a timer watcher that will fire after *after* seconds.

        If *repeat* is given, the timer will continue to fire every *repeat* seconds.
        """
        return TimerWatcher(self, after, ref)

    def signal(self, signum, ref=True, priority=None):
        """
        Create and return a signal watcher for the signal *signum*,
        one of the constants defined in :mod:`signal`.

        This is platform and event loop specific.
        """
        return SignalWatcher(self, signum, ref=ref)

    def idle(self, ref=True, priority=None):
        """
        Create and return a watcher that fires when the event loop is idle.
        """
        pass

    def prepare(self, ref=True, priority=None):
        """
        Create and return a watcher that fires before the event loop
        polls for IO.

        .. caution:: This method is not supported by libuv.
        """
        pass

    def check(self, ref=True, priority=None):
        """
        Create and return a watcher that fires after the event loop
        polls for IO.
        """
        pass

    def fork(self, ref=True, priority=None):
        """
        Create a watcher that fires when the process forks.

        Availability: Unix.
        """
        return ForkWatcher(self, ref=ref)

    def async_(self, ref=True, priority=None):
        """
        Create a watcher that fires when triggered, possibly
        from another thread.

        .. versionchanged:: 1.3
           This was previously just named ``async``; for compatibility
           with Python 3.7 where ``async`` is a keyword it was renamed.
           On older versions of Python the old name is still around, but
           it will be removed in the future.
        """
        return AsyncWatcher(self, ref=ref)

    if sys.platform != "win32":

        def child(self, pid, trace=0, ref=True):
            """
            Create a watcher that fires for events on the child with process ID *pid*.

            This is platform specific and not available on Windows.

            Availability: Unix.
            """
            return ChildWatcher(self, pid, ref=ref)

    def stat(self, path, interval=0.0, ref=True, priority=None):
        """
        Create a watcher that monitors the filesystem item at *path*.

        If the operating system doesn't support event notifications
        from the filesystem, poll for changes every *interval* seconds.
        """

    def run_callback(self, func, *args):
        """
        Run the *func* passing it *args* at the next opportune moment.

        The next opportune moment may be the next iteration of the event loop,
        the current iteration, or some other time in the future.

        Returns a :class:`ICallback` object. See that documentation for
        important caveats.

        .. seealso:: :meth:`asyncio.loop.call_soon`
           The :mod:`asyncio` equivalent.
        """
        return Callback(self, func, args)

    def run_callback_threadsafe(self, func, *args):
        """
        Like :meth:`run_callback`, but for use from *outside* the
        thread that is running this loop.

        This not only schedules the *func* to run, it also causes the
        loop to notice that the *func* has been scheduled (e.g., it causes
        the loop to wake up).

        .. versionadded:: 21.1.0

        .. seealso:: :meth:`asyncio.loop.call_soon_threadsafe`
           The :mod:`asyncio` equivalent.
        """
        return Callback(self, func, args, thread_safe=True)

    def handle_error(self, context, _type, value, tb):
        error_handler = self.error_handler
        if error_handler is not None:
            # we do want to do getattr every time so that setting
            # Hub.handle_error property just works
            handle_error = getattr(error_handler, "handle_error", error_handler)
            handle_error(context, _type, value, tb)
        else:
            self.aio.default_exception_handler(context)

    def _handle_aio_error(self, loop, context):
        try:
            self.handle_error(context, *sys.exc_info())
        except:  # noqa: E722
            # hmm, test__pool will fail if we let the error propagate - it will
            # stop the main loop unexpectedly
            pass

    def reinit(self):
        for watcher in self.fork_watchers:
            self.run_callback(watcher.callback, *watcher.args)

    def install_sigchld(self):
        pass

    def increase_ref(self):
        self._ref_count += 1
        if self._stop_handle is not None:
            self._stop_handle.cancel()
            self._stop_handle = None

    def decrease_ref(self):
        self._ref_count -= 1
        if self._ref_count <= 0 and self._stop_handle is None:
            self._stop_handle = self.aio.call_soon_threadsafe(self._stop)

    def _stop(self):
        self._stop_handle = None
        if self._ref_count <= 0:
            self.aio.stop()

    @property
    def aio(self):
        self._get_or_create_aio()
        return self._aio

    def _get_or_create_aio(self):
        if self._aio and (self._aio.is_closed or self.aio._stopping):
            self._aio = None

        if self._aio:
            return self._aio

        with MonkeyJail():
            import asyncio

            try:
                self._aio = asyncio.get_running_loop()
            except RuntimeError:
                self._aio = asyncio.new_event_loop()
                asyncio.set_event_loop(self._aio)

        return self._aio

    def _format(self):
        return "GeventLoop"



from .watcher import Watcher

__all__ = ["ChildWatcher"]


class ChildWatcher(Watcher):
    def __init__(self, loop, pid, ref=True):
        super().__init__(loop, ref=ref)
        self.pid = pid
        self.watcher = self.loop.policy.get_child_watcher()
        self.rcode = None
        self.rpid = None

    def _start(self):
        self.watcher.add_child_handler(self.pid, self._invoke_wrapper)
        return True

    def _stop(self):
        self.watcher.remove_child_handler(self.pid)

    def _invoke_wrapper(self, pid, retcode):
        self.rpid = pid
        self.rcode = retcode
        self._invoke()



from .watcher import Watcher

__all__ = ["SignalWatcher"]


class SignalWatcher(Watcher):
    def __init__(self, loop, signum, ref=True):
        super().__init__(loop, ref=ref)
        self.signum = signum

    def _start(self):
        self.loop.aio.add_signal_handler(self.signum, self._invoke)
        return True

    def _stop(self):
        self.loop.aio.remove_signal_handler(self.signum)



import asyncio

import gevent.selectors

__all__ = ["EventLoop"]


class EventLoop(asyncio.SelectorEventLoop):
    """
    An asyncio event loop that uses gevent for scheduling and runs in a spawned
    greenlet
    """

    def __init__(self, selector=None):
        super().__init__(selector or gevent.selectors.DefaultSelector())

    def run_forever(self):
        greenlet = gevent.spawn(super(EventLoop, self).run_forever)
        greenlet.join()



from typing import Callable
from typing import Optional

from .future_to_greenlet import future_to_greenlet


def async_to_sync(
    coroutine: Optional[Callable] = None,
    autostart_future: bool = True,
    autocancel_future: bool = True,
    autokill_greenlet: bool = True,
):
    """
    Wrap a coroutine function in a blocking function that spawns a greenlet and blocks until the future is done.
    """
    if coroutine is None:

        def decorator(coroutine: Callable):
            return async_to_sync(
                coroutine=coroutine,
                autostart_future=autostart_future,
                autocancel_future=autocancel_future,
                autokill_greenlet=autokill_greenlet,
            )

        return decorator

    def fn(*args, **kwargs):
        greenlet = future_to_greenlet(
            coroutine(*args, **kwargs),
            autostart_future=autostart_future,
            autocancel_future=autocancel_future,
            autokill_greenlet=autokill_greenlet,
        )
        greenlet.start()
        greenlet.join()
        return greenlet.get()

    return fn



__all__ = ["RefMixin"]


class RefMixin:
    def __init__(self, loop, ref=True):
        self.loop = loop
        self.ref = ref
        self._ref_increased = False

    def _increase_ref(self):
        if self.ref:
            self._ref_increased = True
            self.loop.increase_ref()

    def _decrease_ref(self):
        if self._ref_increased:
            self._ref_increased = False
            self.loop.decrease_ref()

    def __del__(self):
        self._decrease_ref()



import sys
from types import ModuleType
from typing import Dict

__all__ = ["MonkeyJail"]

_sys_modules: Dict[str, ModuleType] = {}


class MonkeyJail:
    def __init__(self):
        self.saved = {}

    def __enter__(self):
        from gevent import monkey

        for key in list(monkey.saved) + ["selectors"]:
            if key in sys.modules:
                self.saved[key] = sys.modules.pop(key)
        sys.modules.update(_sys_modules)

    def __exit__(self, exc_type, exc_val, exc_tb):
        for key in list(self.saved) + ["selectors"]:
            if key in sys.modules:
                _sys_modules[key] = sys.modules[key]
        sys.modules.update(self.saved)



from .async_to_sync import async_to_sync
from .event_loop import EventLoop
from .event_loop_policy import EventLoopPolicy
from .future_to_greenlet import future_to_greenlet
from .greenlet_to_future import greenlet_to_future
from .sync_to_async import sync_to_async

# from .gevent_loop import GeventLoop

__all__ = [
    "async_to_sync",
    "EventLoop",
    "EventLoopPolicy",
    "future_to_greenlet",
    # "GeventLoop",
    "greenlet_to_future",
    "sync_to_async",
]



from typing import Callable
from typing import Optional

import gevent

from .greenlet_to_future import greenlet_to_future


def sync_to_async(
    fn: Optional[Callable] = None,
    autocancel_future: bool = True,
    autostart_greenlet: bool = True,
    autokill_greenlet: bool = True,
) -> Callable:
    """
    Convert a synchronous/blocking function to an asynchronous one.

    This wraps the blocking function `fn` (that may spawn greenlets) in a coroutine function that spawns a greenlet to
    execute `fn` and returns when the greenlet is dead.
    """
    if fn is None:

        def decorator(fn: Callable):
            return sync_to_async(
                fn=fn,
                autocancel_future=autocancel_future,
                autostart_greenlet=autostart_greenlet,
                autokill_greenlet=autokill_greenlet,
            )

        return decorator

    async def coroutine(*args, **kwargs):
        return await greenlet_to_future(
            gevent.Greenlet(fn, *args, **kwargs),
            autocancel_future=autocancel_future,
            autostart_greenlet=autostart_greenlet,
            autokill_greenlet=autokill_greenlet,
        )

    return coroutine



import abc
import sys

from gevent._interfaces import IWatcher
from zope.interface import implementer

from .ref_mixin import RefMixin

__all__ = ["Watcher"]


@implementer(IWatcher)
class Watcher(RefMixin, metaclass=abc.ABCMeta):
    def __init__(self, loop, ref=True):
        super().__init__(loop, ref)
        self._callback = None
        self.args = ()
        self.pending = False
        self.active = False

    @property
    def callback(self):
        return self._callback

    @callback.setter
    def callback(self, callback):
        if not callable(callback) and callback is not None:
            raise TypeError("Expected callable, not %r" % (callback,))
        self._callback = callback

    def start(self, callback, *args, **kwargs):
        if self.active:
            return
        if callback is None:
            raise TypeError("callback must be callable, not None")
        self.callback = callback
        self.args = args
        self.active = self._start(**kwargs)
        if self.active:
            self._increase_ref()

    @abc.abstractmethod
    def _start(self, **kwargs):
        pass

    def stop(self):
        self._decrease_ref()
        self._callback = None
        self.args = None
        self.active = False
        self.pending = False
        self._stop()

    def _stop(self):
        pass

    def _invoke(self):
        self.pending = False
        # noinspection PyBroadException
        try:
            # noinspection PyCallingNonCallable
            self.callback(*self.args)
        except Exception:
            raise
        except:  # noqa: E722
            self.loop.handle_error(self, *sys.exc_info())



from .watcher import Watcher

__all__ = ["TimerWatcher"]


class TimerWatcher(Watcher):
    def __init__(self, loop, after, ref=True):
        super().__init__(loop, ref=ref)
        self.after = after
        self._handle = None

    def _start(self, update=True):
        self._handle = self.loop.aio.call_later(self.after, self._invoke)
        return True

    def _stop(self):
        if self._handle is not None:
            self._handle.cancel()

    def _invoke(self):
        self.active = False
        super()._invoke()

    def __enter__(self):
        self._start()
        return self

    def __exit__(self, *args):
        self._stop()



from .watcher import Watcher

__all__ = ["ForkWatcher"]


class ForkWatcher(Watcher):
    def _start(self):
        self.loop.fork_watchers.add(self)
        return True

    def _stop(self):
        self.loop.fork_watchers.discard(self)



# gevent running on asyncio
# currently _not_ working yet
from .async_watcher import AsyncWatcher
from .callback import Callback
from .child_watcher import ChildWatcher
from .fork_watcher import ForkWatcher
from .gevent_loop import GeventLoop
from .io_watcher import IoWatcher
from .monkey_jail import MonkeyJail
from .ref_mixin import RefMixin
from .signal_watcher import SignalWatcher
from .timer_watcher import TimerWatcher
from .watcher import Watcher

__all__ = [
    "AsyncWatcher",
    "Callback",
    "ChildWatcher",
    "ForkWatcher",
    "GeventLoop",
    "IoWatcher",
    "MonkeyJail",
    "RefMixin",
    "SignalWatcher",
    "TimerWatcher",
    "Watcher",
]



import sys

from gevent._interfaces import ICallback
from zope.interface import implementer

from .ref_mixin import RefMixin

__all__ = ["Callback"]


@implementer(ICallback)
class Callback(RefMixin):
    def __init__(self, loop, callback, args, thread_safe=False):
        super().__init__(loop)
        self.callback = callback
        self.args = args
        self._handle = (
            self.loop.aio.call_soon_threadsafe(self.run)
            if thread_safe
            else self.loop.aio.call_soon(self.run)
        )
        self._increase_ref()

    def stop(self):
        self.callback = None
        self.args = None
        self._handle.cancel()
        self._decrease_ref()

    def run(self):
        try:
            callback, args = self.callback, self.args
            self.callback = self.args = None
            # noinspection PyCallingNonCallable,PyArgumentList
            callback(*args)
        except Exception:
            raise
        except:  # noqa: E722
            self.loop.handle_error(self, *sys.exc_info())
        finally:
            self._decrease_ref()

    def __bool__(self):
        return self.args is not None

    @property
    def pending(self):
        return self.callback is not None

    def close(self):
        pass



import asyncio

import gevent

__all__ = ["greenlet_to_future"]


def _dead_greenlet_to_future(
    greenlet: gevent.Greenlet, future: asyncio.Future, autocancel_future: bool
) -> None:
    try:
        result = greenlet.get(block=False)
        if autocancel_future and isinstance(result, gevent.GreenletExit):
            future.cancel()
        else:
            future.set_result(result)
    except Exception as e:
        future.set_exception(e)


async def _await_greenlet(
    greenlet: gevent.Greenlet,
    autocancel_future: bool,
    autostart_greenlet: bool,
    autokill_greenlet: bool,
):
    # Start the greenlet if it is not yet running
    if not greenlet and autostart_greenlet:
        greenlet.start()

    future: asyncio.Future = asyncio.Future()

    # If the greenlet is dead, set the result

    if greenlet.dead:
        _dead_greenlet_to_future(greenlet, future, autocancel_future)
    else:

        def cb(_):
            _dead_greenlet_to_future(greenlet, future, autocancel_future)

        greenlet.link(cb)

        greenlet.join()

    try:
        return await future
    except asyncio.CancelledError:
        if autokill_greenlet:
            greenlet.kill()
        raise


def greenlet_to_future(
    greenlet: gevent.Greenlet,
    autocancel_future: bool = True,
    autostart_greenlet: bool = True,
    autokill_greenlet: bool = True,
) -> asyncio.Future:
    """
    Wrap a greenlet in a future.

    If the greenlet is already dead when the future is awaited/scheduled, then
    the future will resolve with the result or raise the exception thrown
    immediately.

    If the greenlet is not yet running, the greenlet will by default be started
    when the future is awaited/scheduled. This is to ensure a sensible default
    behaviour and prevent odd concurrency issues. To prevent this
    auto-starting, you can pass `autostart_greenlet=False` as an argument to
    `greenlet_to_future`.

    When a greenlet is killed without a custom exception type, it will return
    (*not* raise) a `GreenletExit` exception. In this instance, by default, the
    future gets cancelled. To circumvent this behaviour and have the function
    return the `GreenletExit` object, you can pass `autocancel_future=False` as
    an argument to `greenlet_to_future`. If a custom exception type is used,
    the future will always raise the exception.

    If the future gets cancelled, then by default the greenlet is killed. To
    prevent the greenlet from getting killed, you can pass
    `autokill_greenlet=False` as an argument to `greenlet_to_future`.
    """

    return asyncio.ensure_future(
        _await_greenlet(
            greenlet, autocancel_future, autostart_greenlet, autokill_greenlet
        )
    )



from .watcher import Watcher

__all__ = ["IoWatcher"]

READ = 1
WRITE = 2


class IoWatcher(Watcher):
    def __init__(self, loop, fd, events, ref=True, priority=None):
        super().__init__(loop, ref=ref)
        self.fd = fd
        self.events = events
        self._reader = events & READ
        self._writer = events & WRITE

    def _start(self, pass_events=False):
        if self._reader:
            self.loop.aio.add_reader(self.fd, self._invoke)
        if self._writer:
            self.loop.aio.add_writer(self.fd, self._invoke)
        return True

    def _stop(self):
        if self._reader:
            self.loop.aio.remove_reader(self.fd)
        if self._writer:
            self.loop.aio.remove_writer(self.fd)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['asyncio_gevent', 'asyncio_gevent.gevent_loop']

package_data = \
{'': ['*']}

extras_require = \
{':python_version >= "3.10" and python_version < "4.0"': ['gevent>=21.12.0,<22.0.0'],
 ':python_version >= "3.7" and python_version < "4.0"': ['gevent']}

setup_kwargs = {
    'name': 'asyncio-gevent',
    'version': '0.2.1',
    'description': 'asyncio & gevent in harmony',
    'long_description': '# asyncio-gevent\n\nasyncio-gevent makes asyncio and gevent compatible. It provides utilities for\n\n- running asyncio on gevent (by using gevent as asyncio\'s event loop)\n- running gevent on asyncio (by using asyncio as gevent\'s event loop, still work in progress)\n- converting greenlets to asyncio futures\n- converting futures to asyncio greenlets\n- wrapping blocking or spawning functions in coroutines which spawn a greenlet and wait for its completion\n- wrapping coroutines in spawning functions which block until the future is resolved\n\nasyncio-gevent is a fork and rewrite of `aiogevent` and `tulipcore` for modern python 3.\n\n## Install\n\nInstall `asyncio-gevent` from pypi using your favourite package manager.\n\n```sh\n# If you use poetry\npoetry add asyncio-gevent\n\n# If you use pip\npip install asyncio-gevent\n```\n\n## Usage\n\n### Running asyncio on gevent\n\nIn order to run `asyncio` on `gevent`, we need to set the (default) `EventLoopPolicy` to use `asyncio_gevent.EventLoopPolicy`.\n\n```py3\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nasyncio.set_event_loop_policy(asyncio_gevent.EventLoopPolicy())\n\nasync def main():\n    await asyncio.sleep(1)\n    print("done")\n\nasyncio.run(main())\n```\n\nAfter setting the event loop policy, asyncio will use an event loop that uses greenlets for scheduling.\n\nUnder the hood, it uses the default selector-based event loop in asyncio with the the gevent selector implementation.\n\nAlternatively, you can also manually set and use the event loop.\n\n```py3\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nloop = asyncio_gevent.EventLoop()\nasyncio.set_event_loop(loop)\n\nasync def main():\n    await asyncio.sleep(1)\n    print("done")\n\nloop.run_until_complete(main())\n```\n\n### Running gevent on asyncio\n\n> This implementation is still work-in-progress. It may work for simple examples, but otherwise fail in unexpected ways.\n\nIn order to run `gevent` on `asyncio`, `gevent` needs to be initialised to use the asyncio event loop. This is done by setting the environment variable `GEVENT_LOOP` to `asyncio_gevent.gevent_loop.GeventLoop` and then starting python.\n\n```sh\nGEVENT_LOOP=asyncio_gevent.gevent_loop.GeventLoop python3 myscript.py\n```\n\ngevent will now run on asyncio.\n\nAlternatively, you can also set the loop configuration setting, preferably right after importing `gevent` and before monkey patching.\n\n```py3\nimport gevent\ngevent.config.loop = "asyncio_gevent.gevent_loop.GeventLoop"\n```\n\n### Converting greenlets to asyncio futures\n\nUse `asyncio_gevent.greenlet_to_future` to convert a greenlet to an asyncio future. The future yields once the greenlet has finished execution.\n\n```py3\n# Preamble: Apply the gevent monkey patch and initialise the asyncio event loop policy\n\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nasyncio.set_event_loop_policy(asyncio_gevent.EventLoopPolicy())\n\n# Main example\n\nimport gevent\n\ndef blocking_function() -> int:\n    gevent.sleep(10)\n    return 42\n\n\nasync def main() -> None:\n    greenlet = gevent.spawn(blocking_function)\n    future = asyncio_gevent.greenlet_to_future(greenlet)\n    result = await future\n\nasyncio.run(main())\n```\n\nIf the greenlet is already dead when the future is awaited/scheduled, then the future will resolve with the result or raise the exception thrown immediately.\n\nIf the greenlet is not yet running, the greenlet will by default be started when the future is awaited/scheduled. This is to ensure a sensible default behaviour and prevent odd concurrency issues. To prevent this auto-starting, you can pass `autostart_greenlet=False` as an argument to `greenlet_to_future`.\n\nWhen a greenlet is killed without a custom exception type, it will return a `GreenletExit` exception. In this instance, the future get cancelled. If a custom exception type is used, the future will raise the exception.\n\nIf the future gets cancelled, then by default the greenlet is killed. To prevent the greenlet from getting killed, you can pass `autokill_greenlet=False` as an argument to `greenlet_to_future`.\n\n### Converting asyncio futures to greenlets\n\nUse `asyncio_gevent.future_to_greenlet` to convert a future to a greenlet.\n\n```py3\n# Preamble: Apply the gevent monkey patch and initialise the asyncio event loop policy\n\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nasyncio.set_event_loop_policy(asyncio_gevent.EventLoopPolicy())\n\n# Main example\n\nimport gevent\n\nasync def async_function() -> int:\n    await asyncio.sleep(1)\n    return 42\n\n\ndef main() -> None:\n    future = async_function()\n    greenlet = asyncio_gevent.future_to_greenlet(future)\n    greenlet.start()\n    greenlet.join()\n    assert greenlet.get() == 42\n\nmain()\n```\n\nThe greenlet returned by this function will not start automatically, so you need to call `Greenlet.start()` manually.\n\nIf `future` is a coroutine object, it will be scheduled as a task on the `loop` when the greenlet starts. If no `loop` argument has been passed, the running event loop will be used. If there is no running event loop, a new event loop will be started using the current event loop policy.\n\nIf the future is not already scheduled, then it won\'t be scheduled for execution until the greenlet starts running. To prevent the future from being scheduled automatically, you can pass `autostart_future=False` as an argument to `future_to_greenlet`.\n\nIf the greenlet gets killed, the by default the future gets cancelled. To prevent this from happening and having the future return the `GreenletExit` objct instead, you can pass `autocancel_future=False` as an argument to `future_to_greenlet`.\n\nIf the future gets cancelled, the greenlet gets killed and will return a `GreenletExit`. This default behaviour can be circumvented by passing `autokill_greenlet=False` and the greenlet will raise the `CancelledError` instead.\n\n### Wrapping blocking or spawning functions in coroutines\n\nUse `asyncio_gevent.sync_to_async` to wrap a blocking function or a function that may spawn greenlets and wait for their completion in a coroutine.\n\n```py3\n# Preamble: Apply the gevent monkey patch and initialise the asyncio event loop policy\n\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nasyncio.set_event_loop_policy(asyncio_gevent.EventLoopPolicy())\n\n# Main example\n\nimport gevent\n\ndef blocking_function(duration: float):\n    gevent.sleep(duration)\n    return 42\n\nasync_function = asyncio_gevent.sync_to_async(blocking_function)\n\nasyncio.run(async_function(1.0))\n```\n\nThe returned corountine function will execute the original function in a spawned greenlet and await it\'s completion.\n\nUnder the hood, this is just a thin convenience wrapper around `asyncio_gevent.greenlet_to_future`.\n\nAs a result, `asyncio_gevent.sync_to_async` accepts the same arguments as `asyncio_gevent.greenlet_to_future` to achieve the same behaviour.\n\n`asyncio_gevent.sync_to_async` can also be used as a decorator\n\n```py3\n# Preamble: Apply the gevent monkey patch and initialise the asyncio event loop policy\n\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nasyncio.set_event_loop_policy(asyncio_gevent.EventLoopPolicy())\n\n# Main example\n\nimport gevent\n\n@asyncio_gevent.sync_to_async()\ndef fn(duration: float):\n    gevent.sleep(duration)\n    return 42\n\nasyncio.run(fn(1.0))\n```\n\n\n### Wrapping coroutines in spawning functions\n\nUse `asyncio_gevent.async_to_sync` to wrap a coroutine function or in a blocking function that spawns a greenlet and waits until the coroutine has returned.\n\n```py3\n# Preamble: Apply the gevent monkey patch and initialise the asyncio event loop policy\n\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nasyncio.set_event_loop_policy(asyncio_gevent.EventLoopPolicy())\n\n# Main example\n\nimport gevent\n\nasync def async_function(duration: float):\n    await asyncio.sleep(duration)\n    return 42\n\nblocking_function = asyncio_gevent.async_to_sync(async_function)\n\nblocking_function(1)\n```\n\nThe returned function will execute the coroutine on an existing running loop or a new event loop and await it\'s completion.\n\nUnder the hood, this is just a thin convenience wrapper around `asyncio_gevent.future_to_greenlet`.\n\nAs a result, `asyncio_gevent.async_to_sync` accepts the same arguments as `asyncio_gevent.future_to_greenlet` to achieve the same behaviour.\n\n`asyncio_gevent.async_to_sync` can also be used as a decorator.\n\n\n```py3\n# Preamble: Apply the gevent monkey patch and initialise the asyncio event loop policy\n\nimport gevent.monkey\ngevent.monkey.patch_all()\n\nimport asyncio\n\nimport asyncio_gevent\n\nasyncio.set_event_loop_policy(asyncio_gevent.EventLoopPolicy())\n\n# Main example\n\nimport gevent\n\n@asyncio_gevent.async_to_sync\nasync def fn(duration: float):\n    await asyncio.sleep(duration)\n    return 42\n\nfn(1)\n```\n\n## License\n\n[MIT](LICENSE)\n',
    'author': 'Frédérique Mittelstaedt',
    'author_email': 'hi@gfm.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gfmio/asyncio-gevent',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from .watcher import Watcher

__all__ = ["AsyncWatcher"]


class AsyncWatcher(Watcher):
    def __init__(self, loop, ref=True):
        super().__init__(loop, ref=ref)
        self._handle = None

    def _start(self):
        return True

    def _stop(self):
        if self._handle is not None:
            self._handle.cancel()

    def send(self):
        self.pending = True
        self._handle = self.loop.aio.call_soon_threadsafe(self._invoke)
